/*

  Copyright (C) 1999 President and Fellows of Harvard College

  All rights reserved.

  NOTICE: This software is provided "as is", without any warranty,
  including any implied warranty for merchantability or fitness for a
  particular purpose.  Under no circumstances shall Harvard University
  or its faculty, staff, students or agents be liable for any use of,
  misuse of, or inability to use this software, including incidental
  and consequential damages.

  License is hereby given to use, modify, and redistribute this
  software, in whole or in part, for any commercial or non-commercial
  purpose, provided that the user agrees to the terms of this
  copyright notice, including disclaimer of warranty, and provided
  that this copyright notice, including disclaimer of warranty, is
  preserved in the source code and documentation of anything derived
  from this software.  Any redistributor of this software or anything
  derived from this software assumes responsibility for ensuring that
  any parties to whom wuch a redistribution is made are fully aware of
  the terms of this license and disclaimer.

  Author: Brad Karp, Harvard University EECS, September, 1997

*/

/* a handles table entry */
struct hhtabent {
  char handle[18];                        /* up to 18 chars; no padding req */
  int handlen;                            /* length of handle in chars */
  int state;                              /* state of this entry */
  struct hhtabent *next;
};
typedef struct hhtabent humr_handent;

#if 0
/* linked list head: all known handles */
extern humr_handent *handent_head;
#endif

/* possible handle table entry states */
#define HANDENT_WAITING 0
#define HANDENT_OLD 1

/* a NNARP table entry */
struct hnnarpent {
  struct in_addr ipdst;                   /* network byte order */
  humr_mac_addr macdst;                   /* HOST byte order */
  int state;                              /* state of this entry */
  struct hnnarpent *next;
  struct hnnarpent *actnext;              /* next entry on active list */
};
typedef struct hnnarpent humr_nnarpent;

#if 0
/* hash table: 256 entries, on IP, linear chaining, all NNARP entries */
extern humr_nnarpent *nnarp_table[256];
/* active list of NNARP entries */
extern humr_nnarpent *nnarpent_active;
#endif

/* my own local MAC address */
extern humr_mac_addr mymac;
/* my own IP address */
extern struct in_addr myip;
/* XXX - fallback next hop IP */
extern struct in_addr fallback;

/* time of last callout */
extern struct timeval lastcout;
/* approximate time of last event that hinted modem alive and in StarMode */
extern struct timeval keepalive;

/* possible NNARP table entry states */
#define NNARPENT_REF0 0
#define NNARPENT_REF1 1
#define NNARPENT_REF2 2
#define NNARPENT_REF3 3

int add_neighbor(char *handle);
void timertick(void);
int receive_iam(humr_mac_addr mac, struct in_addr theip, char *src);
/* initialize the NNARP table */
void nnarp_init(void);
int nnarp_add(humr_mac_addr mac, struct in_addr theip);
/* send an "I-am" to the peer with MAC address mac */
int send_iam_mac(humr_mac_addr mac);
/* send an "I-am" to the peer with handle handle */
int send_iam_handle(char *handle, int handlen);
/* look up NNARP entry for a _network_-ordered IP address dip */
humr_nnarpent *nnarp_lookup(struct in_addr dip);
/* reset neighbor iteration */
void nnarp_neighb_reset(void);
/* iterate over neighbors, returning NNARP entry for each */
humr_nnarpent *nnarp_neighb_iter(void);
